// $Id: AAP.cpp,v 1.2 2003/12/03 01:52:30 magh Exp $

/*-------------------------------------------------------------------------
 Author: Matthew Galati (magh@lehigh.edu)

 (c) Copyright 2003 Lehigh University. All Rights Reserved.

 This software is licensed under the Common Public License. Please see 
 accompanying file for terms.    
---------------------------------------------------------------------------*/

#include "AAP.hpp"

#include <fstream>
using namespace std;

/*--------------------------------------------------------------------------*/
AAP::AAP(const string & datafile) : dimension(0), assigncost(0) {

  //Read in the AAP instance from file
  ifstream is(datafile.c_str());
  if(!is){
    string err("AAP_tm : I/O Error - cannot open ");
    err += datafile;
    throw BCP_fatal_error(err.c_str());
  }


  string buffer;
  std::getline(is, buffer, '='); 
  is >> dimension;
  std::getline(is, buffer);
  std::getline(is, buffer);
    
  int n_cols = dimension * dimension * dimension;
  assigncost = new double[n_cols];
  for(int c = 0; c < n_cols; c++)
    is >> assigncost[c];
  
  is.close();
}

/*--------------------------------------------------------------------------*/
AAP::AAP(BCP_buffer & buf){
  unpack(buf);
}

/*--------------------------------------------------------------------------*/
void AAP::pack(BCP_buffer & buf) const{
  //Pack the AAP instance
  buf.pack(dimension);             //pack an int
  buf.pack(assigncost, n_cols());  //pack an array of doubles of length n_cols
}

/*--------------------------------------------------------------------------*/
void AAP::unpack(BCP_buffer & buf){
  //Unpack the AAP instance
  buf.unpack(dimension);
  int num_cols = n_cols();  
  buf.unpack(assigncost, num_cols);
}
