// $Id: AAP.hpp,v 1.2 2003/12/03 01:52:30 magh Exp $

/* ------------------------------------------------------------------------
 Author: Matthew Galati (magh@lehigh.edu)

 (c) Copyright 2003 Lehigh University. All Rights Reserved.

 This software is licensed under the Common Public License. Please see 
 accompanying file for terms.    
---------------------------------------------------------------------------*/

#ifndef _AAP_H
#define _AAP_H

#include "BCP_buffer.hpp"

#include <string>
#include <iostream>
using namespace std;

/*--------------------------------------------------------------------------*/
//Class for storage of instance of Axial Assignment Problem (AAP)

class AAP{
private:
  int dimension;
  double * assigncost;

public:
  AAP() : dimension(0), assigncost(0) {};
  AAP(const string & datafile);
  AAP(BCP_buffer & buf);
  ~AAP() {
    delete [] assigncost;
  }

public:
  void pack(BCP_buffer & buf) const;
  void unpack(BCP_buffer & buf);  
  inline int      getDimension() const { return dimension; }
  inline double * getAssignCost() const { return assigncost; }
  inline int n_cols() const { return dimension * dimension * dimension; }
};

/*--------------------------------------------------------------------------*/
//Helper Functions for AAP indexing
static inline int index3(const int i, const int j, const int k, const int n){
  return (i * n * n) + (j * n) + k;
}

static inline int index2(const int i, const int j, const int n){
  return (i * n) + j;
}

static inline int ijk_i(const int index, const int n){
  return index / (n * n);
}  

static void ijk(const int index, const int n, int & i, int & j, int & k){
  int n_sq = (n * n);
  int jmod = index % n_sq;
  i = index / n_sq;
  j = jmod / n;
  k = jmod % n;
}

static void ij(const int index, const int n, int & i, int & j){
  int jmod = index % n;
  i = index / n;
  j = jmod;
}

static void ijk_print(const int index, const int n, ostream * os = &cout){
  int i, j, k;
  ijk(index, n, i, j, k);
  (*os) << "(" << i << "," << j << "," << k << ") ";
}



#endif
