// $Id: AAP_lp.hpp,v 1.2 2003/12/03 01:52:30 magh Exp $

/* ------------------------------------------------------------------------
 Author: Matthew Galati (magh@lehigh.edu)

 (c) Copyright 2003 Lehigh University. All Rights Reserved.

 This software is licensed under the Common Public License. Please see 
 accompanying file for terms.    
---------------------------------------------------------------------------*/

#ifndef AAP_LP_H
#define AAP_LP_H

#include "BCP_lp_user.hpp"
#include "BCP_parameters.hpp"

#include "AAP_lp_param.hpp"
#include "AAP_var.hpp"
#include "AAP.hpp"

#include <vector>
#include <functional>
using namespace std;

const double tolerance = 0.0001;
const double bigM      = 100000;

/*--------------------------------------------------------------------------*/
class is_less_than{
public:
  // increasing order on second.first, decreasing on second.second
  bool operator()( const pair< int, pair<double,double> > & x, 
		   const pair< int, pair<double,double> > & y){
    if(x.second.first > y.second.first + tolerance)
      return false;
    else{
      if(x.second.first < y.second.first - tolerance)
	return true;
      else{ 
	// if second.first ==, then check second.second 
	// condition, but flip the return to get decreasing
	if(x.second.second > y.second.second + tolerance)
	  return true;
	else return false;
      }
    }
  }
};

/*--------------------------------------------------------------------------*/
//Class for the user-defined LP interface to BCP

class AAP_lp : public BCP_lp_user{
public:
  BCP_parameter_set<AAP_lp_par> lp_par; //parameter list for LP
  vector<AAP_var*> generated_vars;      //variables generated in compute LB
  AAP * aap;                            //storage for instance of AAP

  //stores the mapping from strong branch candidate to original master variable
  map<int, int> cand_to_variable;       

public:
  AAP_lp() : lp_par(), aap(0) {}

  ~AAP_lp() {
    delete aap;
  };    

public:
  virtual void unpack_module_data(BCP_buffer& buf);
  virtual void pack_var_algo(const BCP_var_algo* var, BCP_buffer& buf);
  virtual BCP_var_algo* unpack_var_algo(BCP_buffer& buf);
  virtual void pack_user_data(const BCP_user_data* ud, BCP_buffer& buf);
  virtual BCP_user_data* unpack_user_data(BCP_buffer& buf);

public:
  virtual OsiSolverInterface* initialize_solver_interface();
  virtual void vars_to_cols(const BCP_vec<BCP_cut*>& cuts,
			    BCP_vec<BCP_var*>& vars,       
			    BCP_vec<BCP_col*>& cols,
			    const BCP_lp_result& lpres,
			    BCP_object_origin origin, bool allow_multiple);
  virtual void display_lp_solution(const BCP_lp_result& lpres,
				   const BCP_vec<BCP_var*>& vars,
				   const BCP_vec<BCP_cut*>& cuts,
				   const bool final_lp_solution);
  virtual double compute_lower_bound(const double old_lower_bound,
				     const BCP_lp_result& lpres,
				     const BCP_vec<BCP_var*>& vars,
				     const BCP_vec<BCP_cut*>& cuts);
  virtual void generate_vars_in_lp(const BCP_lp_result& lpres,
				   const BCP_vec<BCP_var*>& vars,
				   const BCP_vec<BCP_cut*>& cuts,
				   const bool before_fathom,
				   BCP_vec<BCP_var*>& new_vars,
				   BCP_vec<BCP_col*>& new_cols);
  virtual void restore_feasibility(const BCP_lp_result& lpres,
				   const std::vector<double*> dual_rays,
				   const BCP_vec<BCP_var*>& vars,
				   const BCP_vec<BCP_cut*>& cuts,
				   BCP_vec<BCP_var*>& vars_to_add,
				   BCP_vec<BCP_col*>& cols_to_add);

public:
  virtual BCP_branching_decision
  select_branching_candidates(const BCP_lp_result& lpres,
			      const BCP_vec<BCP_var*>& vars,
			      const BCP_vec<BCP_cut*>& cuts,
			      const BCP_lp_var_pool& local_var_pool,
			      const BCP_lp_cut_pool& local_cut_pool,
			      BCP_vec<BCP_lp_branching_object*>& cans);
  virtual void set_user_data_for_children(BCP_presolved_lp_brobj* best,
					  const int selected);

public:
  int solve_assignment_problem(double * ap_costv,
			       double * solution,
			       double & objective_value);
  void construct_cost(const double * pi, const bool from_restore,
		      double * ap_costv, map<int,int> & jk_mini);
  bool generate_vars(const double * pi, const bool from_restore);
  void branch_close_to_half(const BCP_lp_result& lpres,
			    const BCP_vec<BCP_var*>& vars,
			    BCP_vec<BCP_lp_branching_object*>& cans);
  void append_branching_vars(const BCP_lp_result& lpres,
			     const BCP_vec<BCP_var*>& vars,
			     const vector< pair<int, pair<double, 
			     double> > > & var_candidates,
			     BCP_vec<BCP_lp_branching_object*>& cans);
};

#endif
