// $Id: AAP_var.hpp,v 1.2 2003/12/03 01:52:30 magh Exp $

/* ------------------------------------------------------------------------
 Author: Matthew Galati (magh@lehigh.edu)

 (c) Copyright 2003 Lehigh University. All Rights Reserved.

 This software is licensed under the Common Public License. Please see 
 accompanying file for terms.    
---------------------------------------------------------------------------*/

#ifndef AAP_VAR_H
#define AAP_VAR_H

#include "BCP_var.hpp"
#include "BCP_buffer.hpp"
#include "AAP.hpp"
#include "CoinHelperFunctions.hpp"

#include <iostream>
#include <vector>
using namespace std;

/*--------------------------------------------------------------------------*/
/* Class for an AAP algorithmic variable (derived from a BCP_var_algo).
   A variable in this context is an point that satisfies the assignment
   constraints over J x K.
*/

class AAP_var : public BCP_var_algo{
public:
  vector<int> ap; //ijk indices that define the AP point (the J x K assignment)
  double cost;    //the cost of the assignment

public:
  AAP_var() : 
    BCP_var_algo(BCP_BinaryVar, 0.0, 0.0, 1.0), ap(0), cost(0.0) {}
  
  AAP_var(const vector<int> & ap_, const double cost_) :
    BCP_var_algo(BCP_BinaryVar, cost_, 0.0, 1.0), ap(ap_), cost(cost_) {} 

  AAP_var(const AAP_var & x) :
    BCP_var_algo(BCP_BinaryVar, x.cost, 0.0, 1.0), ap(x.ap), cost(x.cost) {} 

  AAP_var(BCP_buffer & buf)
    : BCP_var_algo(BCP_BinaryVar, 0.0, 0.0, 1.0), ap(0), cost(0.0) {
    //Constructor given a buffer stream - unpacks the buffer stream 
    //into this's data members
    unpack(buf);
    set_obj(cost); //a method of BCP_var_algo
  }

  ~AAP_var() {};

public:
  void pack(BCP_buffer & buf) const {
    //Pack the variable
    buf.pack(ap).pack(cost);
  }

  void unpack(BCP_buffer & buf){
    //Unpack the variable
    buf.unpack(ap).unpack(cost);
  } 

  void print(const int dimension, ostream * os = &cout) const {
    //Print the variable (AP point) for debugging
    (*os) << " AP Cost: " << cost << endl;
    for(unsigned int p = 0; p < ap.size(); p++){
      (*os) << " index : " << ap[p] << " : ";
      ijk_print(ap[p], dimension, os);
      (*os) << endl;
    }
  }

};

#endif
